/*
 * Sixth 3D engine. Copyright ©2012-2019, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.wireframe;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.line.LineAppearance;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.base.AbstractCompositeShape;

public class Grid3D extends AbstractCompositeShape {

    public Grid3D(final Point3D p1t, final Point3D p2t, final double step,
                  final LineAppearance appearance) {

        super();

        final Point3D p1 = new Point3D(p1t);
        final Point3D p2 = new Point3D(p2t);

        if (p1.x > p2.x) {
            final double tmp = p1.x;
            p1.x = p2.x;
            p2.x = tmp;
        }

        if (p1.y > p2.y) {
            final double tmp = p1.y;
            p1.y = p2.y;
            p2.y = tmp;
        }

        if (p1.z > p2.z) {
            final double tmp = p1.z;
            p1.z = p2.z;
            p2.z = tmp;
        }

        for (double x = p1.x; x <= p2.x; x += step)
            for (double y = p1.y; y <= p2.y; y += step)
                for (double z = p1.z; z <= p2.z; z += step) {

                    final Point3D p3 = new Point3D(x, y, z);

                    if ((x + step) <= p2.x) {
                        final Point3D point3d2 = new Point3D(x + step, y, z);
                        addShape(appearance.getLine(p3, point3d2));
                    }

                    if ((y + step) <= p2.y) {
                        final Point3D point3d3 = new Point3D(x, y + step, z);
                        addShape(appearance.getLine(p3, point3d3));
                    }

                    if ((z + step) <= p2.z) {
                        final Point3D point3d4 = new Point3D(x, y, z + step);
                        addShape(appearance.getLine(p3, point3d4));
                    }

                }
    }
}
